using Gtk;
using Gdk;

public class BeatBox.SideBarRenderer : CellRenderer {
	public TreeModel model;
	public TreeIter iter;
	public bool expanded;
	public bool highlight;
	public static int EXPANDER_SIZE = 8;
	public static int DEPTH_INDENT_SIZE = 8;
	public static int PIXBUF_SIZE = 18;
	
	public SideBarRenderer() {
		expanded = false;
	}
	
	public override void get_size(Widget widget, Rectangle? cell_area, out int x_offset, out int y_offset, out int width, out int height) {
		x_offset = 0;
		y_offset = 0;
		width = -1;
		height = 20;
	}
	
	public override void render(Cairo.Context context, Widget widget, Rectangle background_area, Rectangle cell_area, CellRendererState flags) {
		Gdk.cairo_rectangle (context, background_area);
		context.clip();
		
		StateFlags state;
		RGBA rgba;
		Pango.Layout layout; // for text to draw
		Pixbuf pix;
		string text;
		int children;
		int depth;
		Pixbuf clickable;
		
		model.get(iter, SideBarColumn.COLUMN_PIXBUF, out pix, SideBarColumn.COLUMN_TEXT, out text, SideBarColumn.COLUMN_CLICKABLE, out clickable);
		TreePath path = model.get_path(iter);
		depth = path.get_depth();
		layout = widget.create_pango_layout(text);
		children = model.iter_n_children(iter);
		state = widget.get_state_flags();
		rgba = widget.get_style_context().get_color (state);
		
		int image_width = 0;
		if(depth != 1 && pix != null)
			image_width = 20;
		
		// Draw a blank space for each level of indent
		double start_x = depth * DEPTH_INDENT_SIZE;
		double start_y = cell_area.y + 1;
		
		// Draw picture if it is not level 1
		if(depth != 1 && pix != null) {
			context.move_to(start_x, start_y);
			Gdk.cairo_set_source_pixbuf(context, pix, (int)start_x, (int)start_y);
			start_x += 20.0;
			context.paint();
		}
		
		// Draw text
		context.set_source_rgba (rgba.red, rgba.green, rgba.blue, rgba.alpha);
		context.move_to(start_x, start_y);
		if(children == 0)
			layout.set_text(text, text.length);
		else
			layout.set_markup("<b>" + text + "</b>", -1);
		Pango.cairo_show_layout(context, layout);
		context.fill();
		
		// If there is a clickable pix, draw it at the very end.
		
		
		// If it has children, draw the expander
		// this is for vala 0.14
		//widget.get_style_context().render_expander(context,
		//					cell_area.x + 8 / 2, cell_area.y + 8 / 2, 8.0, 8.0);//expanded ? ExpanderStyle.EXPANDED : ExpanderStyle.COLLAPSED);
		
		// If it is supposed to be highlighted, draw the highlight
		if(highlight) {
			draw_highlight(context, background_area.x, background_area.y, 
							background_area.width, background_area.height);
		}
		
		// this is for newer vala versions
		if(depth == 1) {
			int far_right = cell_area.x + cell_area.width;
			if(expanded)
				widget.get_style_context().set_state(StateFlags.ACTIVE);
			else
				widget.get_style_context().set_state(StateFlags.NORMAL);
			widget.get_style_context().render_expander(context, far_right - EXPANDER_SIZE,
		                                           cell_area.y + 8 / 2, 8.0, 8.0);
		}
		                         
		                         
		context.fill();
	}
	
	void draw_highlight(Cairo.Context context, double x, double y, double width, double height) {
		double aspect        = 1.0;     /* aspect ratio */
		double corner_radius = (height * 2) / 10.0;   /* and corner curvature radius */

		double radius = corner_radius / aspect;
		double degrees = 3.14 / 180.0;

		context.new_sub_path();
		context.arc (x + width - radius, y + radius, radius, -90 * degrees, 0 * degrees);
		context.arc (x + width - radius, y + height - radius, radius, 0 * degrees, 90 * degrees);
		context.arc (x + radius, y + height - radius, radius, 90 * degrees, 180 * degrees);
		context.arc (x + radius, y + radius, radius, 180 * degrees, 270 * degrees);
		context.close_path();
		
		context.set_source_rgba (0.0, 0.3, 1.0, 0.1);
		context.fill_preserve ();
		context.set_source_rgb (0.0, 0.0, 1);
		context.set_line_width (3.0);
		context.stroke ();
	}
}
