/*-
 * Copyright (c) 2011-2012	   Scott Ringwelski <sgringwe@mtu.edu>
 *
 * Originally Written by Scott Ringwelski for BeatBox Music Player
 * BeatBox Music Player: http://www.launchpad.net/beat-box
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gee;
using Gtk;

public class BeatBox.AlbumGrid : GenericGrid {
	
	public AlbumGrid(LibraryManager lm, TreeViewSetup tvs, int rel_id) {
		base(lm, tvs, rel_id);
		
		set_compare_func(compare_func);
		set_value_func(val_func);
		set_search_func(grid_search_func);
		
		button_release_event.connect(button_release);
		drag_begin.connect(on_drag_begin);
		drag_data_get.connect(on_drag_data_get);
		drag_end.connect(on_drag_end);
	}
	
	protected override void updateTreeViewSetup() {
		
	}
	
	protected override void update_sensitivities() {
		
	}
	
	bool button_release(Gdk.EventButton ev) {
		if(ev.button == 1) {
			TreePath path;
			CellRenderer cell;

			get_item_at_pos((int)ev.x, (int)ev.y, out path, out cell);
			
			if(path == null)
				return false;
			
			Album alb = (Album)get_object_from_index(int.parse(path.to_string()));
			parent_wrapper.popup.set_items(alb);
			parent_wrapper.popup.show_yourself();
		}
		
		return false;
	}
	
	/** **********************************************************
	 * Drag and drop support. GenericView is a source for uris and can
	 * be dragged to a playlist in the sidebar. No support for reordering
	 * is implemented yet.
	***************************************************************/
	void on_drag_begin(Gtk.Widget sender, Gdk.DragContext context) {
		dragging = true;
		lw.dragging_from_music = true;
		debug("drag begin\n");

		Gdk.drag_abort(context, Gtk.get_current_event_time());

		if(get_selected_items().length() == 1) {
			drag_source_set_icon_stock(this, Gtk.Stock.DND);
		}
		else if(get_selected_items().length() > 1) {
			drag_source_set_icon_stock(this, Gtk.Stock.DND_MULTIPLE);
		}
		else {
			return;
		}
	}
	
	void on_drag_data_get(Gdk.DragContext context, Gtk.SelectionData selection_data, uint info, uint time_) {
		string[] uris = null;

		foreach(TreePath path in get_selected_items()) {
			Album a = (Album)get_object_from_index(int.parse(path.to_string()));
			foreach(var m in a.get_medias())
				uris += (m.uri);
		}

		if (uris != null)
			selection_data.set_uris(uris);
	}
	
	void on_drag_end(Gtk.Widget sender, Gdk.DragContext context) {
		dragging = false;
		lw.dragging_from_music = false;

		debug("drag end\n");

		//unset_rows_drag_dest();
		Gtk.drag_dest_set(this,
						  Gtk.DestDefaults.ALL,
						  {},
						  Gdk.DragAction.COPY|
						  Gdk.DragAction.MOVE
						  );
	}
	
	/** **************************************************
	 * View search. All lists use same search algorithm 
	 * *************************************************/
	protected void grid_search_func (string search, HashTable<int, GLib.Object> table, ref HashTable<int, GLib.Object> show) {
		int show_index = 0;
		
		for(int i = 0; i < table.size(); ++i) {
			Album m = (Album)table.get(i);
			
			if(search in m.get_album_artist().down() || search in m.get_album().down()) {
				//if((m.get_album_artist().down() == parent_wrapper.artist_filter.down() || parent_wrapper.artist_filter == "")) {
					show.set(show_index++, table.get(i));
				//}
			}
		}
	}
	
	/***********************************************
	 * Grid value func. Returns markup and pix based on Album
	 * ********************************************/
	string TEXT_MARKUP = "<span weight='medium' size='10500'>%s\n</span><span foreground=\"#999\">%s</span>";
	string TOOLTIP_MARKUP = "<span weight='bold' size='10500'>%s</span>\n%s";
	Value val_func (int row, int column, GLib.Object a_o) {
		Album a = (Album)a_o;
		Value val;
		
		if(column == PIXBUF_COLUMN) {
			var pix = lm.get_cover_album_art_from_key(a.get_album_artist(), a.get_album());
			if(pix == null)
				pix = Icons.DEFAULT_ALBUM_ART_PIXBUF;
				
			val = pix;
		}
		else if(column == MARKUP_COLUMN) {
			string album_artist, album;
			
			if(a.get_album().length > 30)
				album = a.get_album().substring(0, 27) + "...";
			else
				album = a.get_album();

			if(a.get_album_artist().length > 25)
				album_artist = a.get_album_artist().substring(0, 22) + "...";
			else
				album_artist = a.get_album_artist();
			
			album = CellDataFunctionHelper.validate_markup(album);
			album_artist = CellDataFunctionHelper.validate_markup(album_artist);
			val = TEXT_MARKUP.printf(album, album_artist);
		}
		else if(column == TOOLTIP_COLUMN) {
			string album = CellDataFunctionHelper.validate_markup(a.get_album());
			string album_artist = CellDataFunctionHelper.validate_markup(a.get_album_artist());
			
			val = TOOLTIP_MARKUP.printf (album, album_artist);
		}
		else {
			val = a;
		}
		
		return val;
	}
	
	int compare_func(GLib.Object o_a, GLib.Object o_b) {
		Album a = (Album)o_a;
		Album b = (Album)o_b;
		
		if(a.get_album_artist() == b.get_album_artist()) {
			if(a.get_album() == b.get_album()) {
				return a.count() - b.count();
			}
			else {
				return advanced_string_compare(a.get_album(), b.get_album());
			}
		}
		else {
			return advanced_string_compare(a.get_album_artist(), b.get_album_artist());
		}
	}
	
	int advanced_string_compare(string a, string b) {
		if(a == "" && b != "")
			return 1;
		else if(a != "" && b == "")
			return -1;
		
		return (a > b) ? 1 : -1;
	}
}

