using Gee;
using Gtk;

public abstract class BeatBox.GenericGrid : FastGrid {
	protected LibraryManager lm;
	protected LibraryWindow lw;
	protected ViewWrapper parent_wrapper;
	
	// For shuffle
	protected int old_sort_col;
	protected SortType old_sort_dir;
	
	protected TreeViewSetup tvs;
	protected int relative_id;
	protected bool is_current_list;
	
	protected bool scrolled_recently;
	protected bool dragging;
	
	protected GLib.Icon playing_icon;
	protected GLib.Icon completed_icon;
	protected GLib.Icon saved_locally_icon;
	protected GLib.Icon new_podcast_icon;
	
	// To select which columns are showing
	protected Gtk.Menu columnChooserMenu;
	
	public signal void import_requested(LinkedList<Media> to_import);
	
	private const string WIDGET_STYLESHEET = """
        .GenericGrid {
            background-color: #FFFFFF;
        }
    """;
	
	public GenericGrid(LibraryManager lm, TreeViewSetup tvs, int rel_id) {
		base();
		
		this.lm = lm;
		this.lw = lm.lw;
		this.tvs = tvs;
		this.relative_id = rel_id;
		
		playing_icon = Icons.MEDIA_PLAY_SYMBOLIC.get_gicon ();
		completed_icon = Icons.PROCESS_COMPLETED.get_gicon ();
		saved_locally_icon = new GLib.ThemedIcon.with_default_fallbacks (Gtk.Stock.SAVE);
		new_podcast_icon = Icons.NEW_PODCAST.get_gicon ();
		
		// Change background color
		var style_provider = new CssProvider();

        try  {
            style_provider.load_from_data (WIDGET_STYLESHEET, -1);
        } catch (Error e) {
            stderr.printf ("\nGenericGrid: Couldn't load style provider.\n");
        }

        get_style_context().add_class("GenericGrid");
        get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		
		// drag source
		TargetEntry te = { "text/uri-list", TargetFlags.SAME_APP, 0};
		drag_source_set(this, Gdk.ModifierType.BUTTON1_MASK, { te }, Gdk.DragAction.COPY);
		//enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, {te}, Gdk.DragAction.COPY);
		
		//vadjustment.value_changed.connect(view_scroll);
		//row_activated.connect(row_activated_signal);
		//rows_reordered.connect(updateTreeViewSetup);
		lm.current_cleared.connect(current_cleared);
		//lm.media_played.connect(media_played);
		//lm.medias_updated.connect(medias_updated);
	}
	
	public void set_parent_wrapper(ViewWrapper parent) {
		this.parent_wrapper = parent;
	}
	
	public abstract void update_sensitivities();
	
	/** TreeViewColumn header functions. Has to do with sorting and
	 * remembering column widths/sort column/sort direction between
	 * sessions.
	**/
	protected abstract void updateTreeViewSetup();
	
	
	void current_cleared() {
		is_current_list = false;
	}
	
	/***************************************
	 * Simple setters and getters
	 * *************************************/
	public void set_hint(ViewWrapper.Hint hint) {
		tvs.set_hint(hint);
	}
	
	public ViewWrapper.Hint get_hint() {
		return tvs.get_hint();
	}
	
	public void set_relative_id(int id) {
		this.relative_id = id;
	}
	
	public int get_relative_id() {
		return relative_id;
	}
	
	public bool get_is_current_list() {
		return is_current_list;
	}
}
