using Gee;
using Gtk;

public abstract class BeatBox.GenericList : FastList {
	protected LibraryManager lm;
	protected LibraryWindow lw;
	protected ViewWrapper parent_wrapper;
	
	// For shuffle
	protected int old_sort_col;
	protected SortType old_sort_dir;
	HashTable<int, Media> unshuffled_cache;
	
	protected TreeViewSetup tvs;
	protected int relative_id;
	protected bool is_current_list;
	
	protected bool scrolled_recently;
	protected bool dragging;
	
	protected CellDataFunctionHelper cellHelper;
	
	protected GLib.Icon playing_icon;
	protected GLib.Icon completed_icon;
	protected GLib.Icon saved_locally_icon;
	protected GLib.Icon new_podcast_icon;
	
	// To select which columns are showing
	protected Gtk.Menu columnChooserMenu;
	protected Gtk.MenuItem browseSame;
	protected Gtk.Menu browseSameMenu;
	protected Gtk.MenuItem browseSameAlbum;
	protected Gtk.MenuItem browseSameArtist;
	protected Gtk.MenuItem browseSameGenre;
	
	public signal void import_requested(LinkedList<Media> to_import);
	
	public GenericList(LibraryManager lm,  GLib.List<Type> types, TreeViewSetup tvs, int rel_id) {
		base(types);
		
		this.lm = lm;
		this.lw = lm.lw;
		this.tvs = tvs;
		this.relative_id = rel_id;
		
		set_headers_clickable(true);
		set_headers_visible(true);
		set_fixed_height_mode(true);
		set_rules_hint(true);
		set_reorderable(false);
		
		unshuffled_cache = new HashTable<int, Media>(null, null);
		
		cellHelper = new CellDataFunctionHelper((BeatBox.LibraryManager)lm, this);
		playing_icon = Icons.MEDIA_PLAY_SYMBOLIC.get_gicon ();
		completed_icon = Icons.PROCESS_COMPLETED.get_gicon ();
		saved_locally_icon = new GLib.ThemedIcon.with_default_fallbacks (Gtk.Stock.SAVE);
		new_podcast_icon = Icons.NEW_PODCAST.get_gicon ();
		
		// drag source
		TargetEntry te = { "text/uri-list", TargetFlags.SAME_APP, 0};
		drag_source_set(this, Gdk.ModifierType.BUTTON1_MASK, { te }, Gdk.DragAction.COPY);
		//enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, {te}, Gdk.DragAction.COPY);
		
		// allow selecting multiple rows
		get_selection().set_mode(SelectionMode.MULTIPLE);
		
		columnChooserMenu = new Gtk.Menu();
		
		/*browseSame = new Gtk.MenuItem.with_label("Show only...");
		browseSameAlbum = new Gtk.MenuItem.with_label("this Album");
		browseSameArtist = new Gtk.MenuItem.with_label("this Artist");
		browseSameGenre = new Gtk.MenuItem.with_label("this Genre");
		browseSameAlbum.activate.connect(browse_same_album_activate);
		browseSameArtist.activate.connect(browse_same_artist_activate);
		browseSameGenre.activate.connect(browse_same_genre_activate);
		browseSameMenu = new Gtk.Menu();
		browseSameMenu.append(browseSameAlbum);
		browseSameMenu.append(browseSameArtist);
		browseSameMenu.append(browseSameGenre);
		browseSame.submenu = browseSameMenu;*/
		
		//vadjustment.value_changed.connect(view_scroll);
		drag_begin.connect(on_drag_begin);
		drag_data_get.connect(on_drag_data_get);
		drag_end.connect(on_drag_end);
		row_activated.connect(row_activated_signal);
		rows_reordered.connect(updateTreeViewSetup);
		lm.current_cleared.connect(current_cleared);
		lm.media_played.connect(media_played);
		lm.medias_updated.connect(medias_updated);
	}
	
	public void set_parent_wrapper(ViewWrapper parent) {
		this.parent_wrapper = parent;
	}
	
	public abstract void update_sensitivities();
	
	/** TreeViewColumn header functions. Has to do with sorting and
	 * remembering column widths/sort column/sort direction between
	 * sessions.
	**/
	protected abstract void updateTreeViewSetup();
	
	protected void add_columns() {
		int index = 0;
		
		foreach(TreeViewColumn tvc in tvs.get_columns()) {
			if(!(tvc.title == " " || tvc.title == "id")) {
				// Music, General
				if(tvc.title == "Bitrate")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.bitrateTreeViewFiller);
				else if(tvc.title == "Length")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.lengthTreeViewFiller);
				else if(tvc.title == "Date Added")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.dateTreeViewFiller);
				else if(tvc.title == "Last Played")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.dateTreeViewFiller);
				else if(tvc.title == "Rating")
					insert_column_with_data_func(-1, tvc.title, new CellRendererPixbuf(), cellHelper.ratingTreeViewFiller);
				else if(tvc.title == "Year")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.intelligentTreeViewFiller);
				else if(tvc.title == "#")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.intelligentTreeViewFiller);
				else if(tvc.title == "Track")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.intelligentTreeViewFiller);
				else if(tvc.title == "Plays")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.intelligentTreeViewFiller);
				else if(tvc.title == "Skips")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.intelligentTreeViewFiller);
				else if(tvc.title == "Title")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.stringTreeViewFiller);
				else if(tvc.title == "Artist")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.stringTreeViewFiller);
				else if(tvc.title == "Album")
					insert_column_with_data_func(-1, tvc.title, new SmartAlbumRenderer(), cellHelper.smartAlbumFiller);
				else if(tvc.title == "Genre")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.stringTreeViewFiller);
				else if(tvc.title == "BPM")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.intelligentTreeViewFiller);
				
				// Podcast
				else if(tvc.title == "Date")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.dateTreeViewFiller);
				else if(tvc.title == "Episode")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.intelligentTreeViewFiller);
				else if(tvc.title == "Name")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.stringTreeViewFiller);
				else if(tvc.title == "Comment")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.stringTreeViewFiller);
				else if(tvc.title == "Category")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.stringTreeViewFiller);
				else if(tvc.title == "Podcast")
					insert_column_with_data_func(-1, tvc.title, new SmartAlbumRenderer(), cellHelper.smartAlbumFiller);
				else if(tvc.title == "Pulser")
					insert_column(tvc, index); // cells for pulser is made ready to go in TVS
				
				// Radio
				else if(tvc.title == "Station")
					insert_column_with_data_func(-1, tvc.title, new CellRendererText(), cellHelper.stringTreeViewFiller);
				
				else {
					warning("Unknown column found. Adding anyways (%s)\n", tvc.title);
					insert_column(tvc, index);
				}


				get_column(index).resizable = true;
				get_column(index).reorderable = false;
				get_column(index).clickable = true;
				get_column(index).sort_column_id = index;
				get_column(index).set_sort_indicator(false);
				get_column(index).visible = tvc.visible;
				get_column(index).sizing = Gtk.TreeViewColumnSizing.FIXED;
				get_column(index).fixed_width = tvc.fixed_width;
			}
			else if(tvc.title == " ") {
				insert_column(tvc, index);

				tvc.fixed_width = 24;
				tvc.clickable = false;
				tvc.sort_column_id = -1;
				tvc.resizable = false;
				tvc.reorderable = false;

				tvc.set_cell_data_func(tvc.get_cells().nth_data(0), cellHelper.iconDataFunc);
				tvc.set_cell_data_func(tvc.get_cells().nth_data(1), cellHelper.iconDataFunc);
			}
			else if(tvc.title == "id") {
				insert_column(tvc, index);
			}
			else {
				insert_column(tvc, index);
			}

			get_column(index).get_button().button_press_event.connect(view_header_click);
			get_column(index).notify["width"].connect(updateTreeViewSetup);

			++index;
		}
	}
	
	protected bool view_header_click(Gtk.Widget w, Gdk.EventButton e) {
		if(e.button == 3) {
			columnChooserMenu.popup (null, null, null, 3, get_current_event_time());
			return true;
		}
		else if(e.button == 1) {
			// If the user tries to sort, then make sure that all other views
			// become sorted (unshuffled) if they are shuffled
			if(lw.shuffleChooser.get_selected_index() == (int)LibraryManager.Shuffle.ALL) {
				lw.shuffleChooser.setOption(0);
				return true;
			}
			
			updateTreeViewSetup();
			
			return false;
		}

		return false;
	}
	
	void row_activated_signal(TreePath path, TreeViewColumn column) {
		if(tvs.get_hint() == ViewWrapper.Hint.DEVICE_AUDIO || tvs.get_hint() == ViewWrapper.Hint.DEVICE_PODCAST) {
			lw.doAlert("Playing not Supported", "Due to issues with playing songs on certain iOS devices, playing songs off devices is currently not supported.");
			return;
		}
		
		Media m = get_media_from_index(int.parse(path.to_string()));
		
		// We need to first set this as the current list
		lm.clearCurrent();
		is_current_list = true;
		
		// Now update current_list and current_index in LM
		set_as_current_list(m);
		
		// Now play the song
		lm.playMedia(m, false);
		
		if(!lm.playing) {
			lw.playClicked();
		}
	}
	
	void media_played(Media m, Media? old) {
		// find index of given media
		int old_index = -1;
		int id_index = -1;
		for(int i = 0; i < get_visible_table().size(); ++i) {
			if(old != null && get_visible_table().get(i) == old) {
				old_index = i;
				if(id_index != -1) break;
			}
			else if(get_visible_table().get(i) == m) {
				id_index = i;
				if(old_index != -1 || old == null) break;
			}
		}
		
		if(old_index != -1)		redraw_row(old_index);
		if(id_index != -1)		redraw_row(id_index);

		if(!scrolled_recently) {
			//scroll_to_current_media();
		}
	}
	
	public void medias_updated(LinkedList<int> ids) {
		var map = new HashMap<int, int>();
		foreach(var i in ids)
			map.set(i, 1);
		
		for(int i = 0; i < get_visible_table().size(); ++i) {
			if(map.get(get_visible_table().get(i).rowid) == 1) {
				redraw_row (i);
			}
		}
	}
	
	// If turning on shuffle, first create a copy of the current list
	// and save it. This ensure we can unshuffle instantly. Then, save the
	// current sort data for later.
	// If turning off shuffle, use the old saved list and values to restore
	// the unshuffled state.
	public void toggle_shuffle(bool val) {
		if(val && !(sort_column_id == SHUFFLE_COLUMN_ID)) {
			old_sort_col = sort_column_id;
			old_sort_dir = sort_direction;
			
			unshuffled_cache = new HashTable<int, Media>(null, null);
			var table = get_table();
			for(int i = 0; i < table.size(); ++i) {
				unshuffled_cache.set(i, table.get(i));
			}
			
			set_sort_column_id (SHUFFLE_COLUMN_ID, sort_direction);
		}
		else if(!val) {
			//sort_column_id = old_sort_col;
			//sort_direction = old_sort_dir;
			
			var new_table = new HashTable<int, Media>(null, null);
			for(int i = 0; i < unshuffled_cache.size(); ++i) {
				new_table.set(i, unshuffled_cache.get(i));
			}
			set_table(new_table);
			
			set_sort_column_id (old_sort_col, old_sort_dir);
		}
		
		updateTreeViewSetup();
		if(is_current_list) {
			set_as_current_list ();
			scroll_to_current_media (false);
		}
	}
	
	void current_cleared() {
		is_current_list = false;
	}
	
	public void set_as_current_list(Media? m = null) {
		Media to_set;
		if(m != null)
			to_set = m;
		else
			to_set = lm.media_info.media;
		
		lm.clearCurrent();
		is_current_list = true;
		
		lm.current_index = 0;
		var vis_table = get_visible_table();
		for(int i = 0; i < vis_table.size(); ++i) {
			Media test = vis_table.get(i);
			lm.addToCurrent(test);

			
			if(to_set == test) {
				lm.current_index = i;
			}
		}
		
		media_played(lm.media_info.media, lm.media_info.media);
	}
	
	protected GLib.List<Media> get_selected_medias() {
		var rv = new GLib.List<Media>();
		TreeModel temp;
		
		foreach(TreePath path in get_selection().get_selected_rows(out temp)) {
			Media m = get_media_from_index(int.parse(path.to_string()));
			rv.append(m);
		}
		
		return rv;
	}
	
	/*void view_scroll() {
		if(!scrolled_recently) {
			Timeout.add(30000, () => {
				scrolled_recently = false;

				return false;
			});

			scrolled_recently = true;
		}
	}*/
	
	protected void mediaScrollToCurrentRequested() {
		scroll_to_current_media(true);
	}
	
	public void scroll_to_current_media(bool unfilter_if_not_found) {
		if(!visible || lm.media_info.media == null)
			return;
		
		for(int i = 0; i < get_visible_table().size(); ++i) {
			Media m = get_media_from_index(i);

			if(m.rowid == lm.media_info.media.rowid) {
				scroll_to_cell(new TreePath.from_string(i.to_string()), null, false, 0.0f, 0.0f);
				scrolled_recently = false;

				return;
			}
		}
		
		if(unfilter_if_not_found) {
			// At this point, it was not scrolled to. Let's see if it's in ALL the songs
			// and if so, undo the search and filters and scroll to it.
			var whole_table = get_table();
			for(int i = 0; i < whole_table.size(); ++i) {
				Media m = whole_table.get(i);

				if(m.rowid == lm.media_info.media.rowid) {
					// Undo search and filter
					parent_wrapper.clear_filters();
					lw.searchField.set_text("");
					
					// And now scroll to it.
					scroll_to_cell(new TreePath.from_string(i.to_string()), null, false, 0.0f, 0.0f);
					scrolled_recently = false;

					return;
				}
			}
		}
		
		scrolled_recently = false;
	}
	
	/***************************************
	 * Simple setters and getters
	 * *************************************/
	public void set_hint(ViewWrapper.Hint hint) {
		tvs.set_hint(hint);
	}
	
	public ViewWrapper.Hint get_hint() {
		return tvs.get_hint();
	}
	
	public void set_relative_id(int id) {
		this.relative_id = id;
	}
	
	public int get_relative_id() {
		return relative_id;
	}
	
	public bool get_is_current_list() {
		return is_current_list;
	}
	
	/** **********************************************************
	 * Drag and drop support. GenericView is a source for uris and can
	 * be dragged to a playlist in the sidebar. No support for reordering
	 * is implemented yet.
	***************************************************************/
	void on_drag_begin(Gtk.Widget sender, Gdk.DragContext context) {
		dragging = true;
		lw.dragging_from_music = true;
		debug("drag begin\n");

		Gdk.drag_abort(context, Gtk.get_current_event_time());

		if(get_selection().count_selected_rows() == 1) {
			drag_source_set_icon_stock(this, Gtk.Stock.DND);
		}
		else if(get_selection().count_selected_rows() > 1) {
			drag_source_set_icon_stock(this, Gtk.Stock.DND_MULTIPLE);
		}
		else {
			return;
		}
	}
	
	void on_drag_data_get(Gdk.DragContext context, Gtk.SelectionData selection_data, uint info, uint time_) {
		string[] uris = null;

		foreach(Media m in get_selected_medias()) {
			uris += (m.uri);
		}

		if (uris != null)
			selection_data.set_uris(uris);
	}
	
	void on_drag_end(Gtk.Widget sender, Gdk.DragContext context) {
		dragging = false;
		lw.dragging_from_music = false;

		debug("drag end\n");

		//unset_rows_drag_dest();
		Gtk.drag_dest_set(this,
						  Gtk.DestDefaults.ALL,
						  {},
						  Gdk.DragAction.COPY|
						  Gdk.DragAction.MOVE
						  );
	}
	
	/** **************************************************
	 * View search. All lists use same search algorithm 
	 * *************************************************/
	protected void view_search_func (string search, HashTable<int, Media> table, ref HashTable<int, Media> show) {
		int show_index = 0;
		
		for(int i = 0; i < table.size(); ++i) {
			Media m = table.get(i);
			
			if(search in m.artist.down() || search in m.album_artist.down() ||
			search in m.album.down() || search in m.title.down() ||
			search in m.genre.down()) {
				if((m.album_artist.down() == parent_wrapper.artist_filter.down() || parent_wrapper.artist_filter == "") &&
				(m.album.down() == parent_wrapper.album_filter.down() || parent_wrapper.album_filter == "") &&
				(m.genre.down() == parent_wrapper.genre_filter.down() || parent_wrapper.genre_filter == "")) {
					show.set(show_index++, table.get(i));
				}
			}
		}
	}
	
	/************************************************
	 * Used by all views to sort list
	 * ******************************************/
	protected int advanced_string_compare(string a, string b) {
		if(a == "" && b != "")
			return 1;
		else if(a != "" && b == "")
			return -1;
		
		return (a > b) ? 1 : -1;
	}
}
