using Gtk;
using Gee;

public class BeatBox.SearchSuggester : Gtk.Menu {
	static const int MAX_STRING_LENGTH = 20;
	LibraryManager lm;
	LibraryWindow lw;
	string last_search;
	bool listen_for_search;
	
	public enum GroupType {
		TOP_MEDIAS,
		TOP_ARTISTS,
		TOP_ALBUMS,
		TOP_GENRES,
		RECENTLY_ADDED,
		RECENTLY_PLAYED,
		RECENT_FAVORITES,
		OLD_FAVORITES,
		POTENTIAL_FAVORITES
	}
	
	public SearchSuggester(LibraryManager lm, LibraryWindow lw) {
		this.lm = lm;
		this.lw = lw;
		
		set_size_request (Icons.ALBUM_VIEW_IMAGE_SIZE, -1);
		key_press_event.connect(keyPressed);
		attach_to_widget(lw.searchField, null);
		lw.searchField.changed.connect(search_field_changed);
	}
	
	public void set_listen_for_search(bool val) {
		listen_for_search = val;
	}
	
	public bool get_listen_for_search() {
		return listen_for_search;
	}
	
	void search_field_changed() {
		if(!listen_for_search)
			return;
		
		var new_search = lw.searchField.get_text();

		if(lw.initializationFinished) {
			last_search = new_search;
			if(new_search == "") {
				clear();
				popdown();
			}
			
			//Timeout.add(750, () => {
				if(new_search == last_search && new_search.length > 2) {
					try {
						Thread.create<void*>(update_contents, false);
					} catch(GLib.ThreadError err) {
						warning ("ERROR: Could not create last fm thread: %s \n", err.message);
					}
				}
				else if(new_search.length <= 2) {
					clear ();
					popdown ();
				}
				
			//	return false;
			//});
		}
	}
	
	public void clear() {
		foreach(var child in get_children()) {
			remove(child);
		}
	}
	
	bool keyPressed(Gdk.EventKey event) {
		if(event.keyval == 0xff08) { //backspace
			lw.searchField.backspace ();
			return false;
		}
		else if(Regex.match_simple("[a-zA-Z0-9]", event.str) || 
		event.str == "-" || event.str == "-" || event.str == "&" ||
		event.str == "(" || event.str == ")" || event.str == "/" ||
		event.str == "." || event.str == "!" || event.str == "?" ||
		event.str == " ") {
			lw.searchField.insert_at_cursor(event.str);
			return false;
		}

		return false;
	}
	
	// go through visible table and find top tracks, albums, artists, genres.
	void* update_contents() {
		Widget current_widget = lw.get_current_view_wrapper ();
		if(!(current_widget is ViewWrapper))
			return null;
		
		ViewWrapper current_wrapper = (ViewWrapper)current_widget;
		
		var table = current_wrapper.list_view.get_table();
		var track_score = new HashMap<int, int>(); // media.rowid, score
		var album_score = new HashMap<string, int>(); // artist - album, score
		var artist_score = new HashMap<string, int>(); // artist, score
		var genre_score = new HashMap<string, int>(); // artist, score
		
		// first calculate scores for each song. this is used to calculate
		// genre, artist, and album scores
		string search = lw.searchField.get_text().down();
		foreach(var m in table.get_values()) {
			var score = calculate_score(m, search);
			
			if(search in m.title.down()) {
				bool has_prefix = m.title.down().has_prefix(search);
				track_score.set(m.rowid, score + ((has_prefix) ? 3 : 0)); // extra score if search is in title of song
			}
			if(search in m.album.down()) {
				//track_score.set(m.rowid, score);
				bool has_prefix = m.album.down().has_prefix(search);
				album_score.set(m.album_artist + "<separator>" + m.album, album_score.get(m.album_artist + "<separator>" + m.album) + score + 2 + ((has_prefix) ? 3 : 0));
			}
			if(search in m.album_artist.down() || search in m.album_artist.down()) {
				//track_score.set(m.rowid, score);
				bool has_prefix = m.album_artist.down().has_prefix(search);
				album_score.set(m.album_artist + "<separator>" + m.album, album_score.get(m.album_artist + "<separator>" + m.album) + score + 1 + ((has_prefix) ? 3 : 0));
				artist_score.set(m.album_artist, artist_score.get(m.album_artist) + score + 1);
			}
			if(m.genre != "" && search in m.genre.down()) {
				//track_score.set(m.rowid, score);
				album_score.set(m.album_artist + "<separator>" + m.album, album_score.get(m.album_artist + "<separator>" + m.album) + score);
				artist_score.set(m.album_artist, artist_score.get(m.album_artist) + score);
				genre_score.set(m.genre, genre_score.get(m.genre) + score);
			}
		}
		
		// Find top 4
		var topTracks = getTopTracks(track_score, 5);
		var topArtists = getTopScores(artist_score, 4);
		var topAlbums = getTopScores(album_score, 5);
		var topGenres = getTopScores(genre_score, 3);
		
		// Convert to LinkedList<Media>
		var tracks = new LinkedList<Media>();
		foreach(var id in topTracks) {
			Media m = lm.media_from_id(id);
			tracks.add(m);
		}
		
		var artists = new LinkedList<Media>();
		foreach(var artist in topArtists) {
			Media m = new Media("");
			m.album_artist = artist;
			artists.add(m);
		}
		
		var albums = new LinkedList<Media>();
		foreach(var album in topAlbums) {
			Media m = new Media("");
			
			string[] data = album.split("<separator>", 0);
			m.album_artist = data[0];
			m.album = data[1];
			
			albums.add(m);
		}
		
		var genres = new LinkedList<Media>();
		foreach(var genre in topGenres) {
			Media m = new Media("");
			m.genre = genre;
			genres.add(m);
		}
		
		Idle.add( () => {
			clear ();
			
			bool results = false;
			if(tracks.size > 0) {
				results = true;
				
				foreach(var m in tracks) {
					var item = create_menu_item(m, GroupType.TOP_MEDIAS);
					append(item);
					item.activate.connect( () => {
						lm.playMedia(m, false);
						if(!lm.playing)
							lw.playClicked ();
						
						current_wrapper.list_view.set_as_current_list();
					});
				}
				
				if(albums.size > 0 || artists.size > 0 || genres.size > 0)
					append(new SeparatorMenuItem());
			}
			if(albums.size > 0) {
				results = true;
				
				foreach(var m in albums) {
					var item = create_menu_item(m, GroupType.TOP_ALBUMS);
					append(item);
					item.activate.connect( () => {
						current_wrapper.album_filter = m.album;
						current_wrapper.artist_filter = m.album_artist;
						
						// User likely does not want to stare at 1 album
						if(lw.viewSelector.selected == 1)
							lw.viewSelector.selected = 0;
					});
				}
				
				if(artists.size > 0 || genres.size > 0)
					append(new SeparatorMenuItem());
			}
			if(artists.size > 0) {
				results = true;
				
				foreach(var m in artists) {
					var item = create_menu_item(m, GroupType.TOP_ARTISTS);
					append(item);
					item.activate.connect( () => {
						current_wrapper.artist_filter = m.album_artist;
					});
				}
				
				if(genres.size > 0)
					append(new SeparatorMenuItem());
			}
			if(genres.size > 0) {
				results = true;
				
				foreach(var m in genres) {
					var item = create_menu_item(m, GroupType.TOP_GENRES);
					append(item);
					item.activate.connect( () => {
						current_wrapper.genre_filter = m.genre;
						
						current_wrapper.list_view.do_search (null);
						current_wrapper.set_statusbar_info();
					});
				}
			}
			
			if(results) {
				show_all();
				popup (null, null, menu_pos_func, 3, get_current_event_time());
			}
			else {
				hide ();
				popdown ();
			}
			
			return false;
		});
		
		return null;
	}
	
	void menu_pos_func(Gtk.Menu menu, out int x, out int y, out bool push_in) {
		int dest_x, dest_y;
		int win_x, win_y;
		lw.searchField.translate_coordinates(lw, 0, lw.searchField.get_allocated_height(), out dest_x, out dest_y);
		lw.get_position(out win_x, out win_y);
		
		x = dest_x + win_x;
		y = dest_y + win_y + 16;
		push_in = true;
	}
	
	Gtk.MenuItem create_menu_item(Media m, GroupType type) {
		Gtk.MenuItem item;
		string markup = "";
		Gdk.Pixbuf? pixbuf = null;
		
		if(type == GroupType.TOP_ALBUMS || type == GroupType.TOP_MEDIAS)
			pixbuf = lm.get_cover_album_art_from_key(m.album_artist, m.album);
		if(pixbuf == null)
			pixbuf = Icons.DEFAULT_ALBUM_ART_PIXBUF;
		
		if(type == GroupType.TOP_MEDIAS) {
			item = new Gtk.ImageMenuItem.with_label("");
			markup = "<span weight='medium' size='10500'>" + validate_markup(m.title) + "</span>" + 
													((m.artist != "") ? ("\n<span foreground=\"#999\">" + validate_markup(m.artist) + "</span>") : "");
		}
		else if(type == GroupType.TOP_ALBUMS) {
			item = new Gtk.ImageMenuItem.with_label("");
			markup = "<span weight='medium' size='10500'>" + validate_markup(m.album) + "</span>" + 
													((m.album_artist != "") ? ("\n<span foreground=\"#999\">" + validate_markup(m.album_artist) + "</span>") : "");
		}
		else if(type == GroupType.TOP_ARTISTS) {
			item = new Gtk.MenuItem.with_label("");
			markup = "<span weight='medium' size='10500'>" + validate_markup(m.album_artist) + "</span>";
		}
		else { // if(type == GroupType.TOP_GENRES) {
			item = new Gtk.MenuItem.with_label("");
			markup = "<span weight='medium' size='10500'>" + validate_markup(m.genre) + "</span>";
		}
		
		foreach(var child in item.get_children()) {
			if(child is Label) {
				((Label)child).set_markup(markup);
			}
		}
		if(item is ImageMenuItem) {
			((ImageMenuItem)item).set_always_show_image(true);
			((ImageMenuItem)item).set_image(new Image.from_pixbuf(pixbuf.scale_simple(32, 32, Gdk.InterpType.BILINEAR)));
		}
		
		
		return item;
	}
	
	string validate_markup(string s) {
		string rv = s;
		
		if(s.length > MAX_STRING_LENGTH)
			rv = s.substring(0, MAX_STRING_LENGTH - 1) + "...";
		
		return rv.replace("&", "&amp;");
	}
	
	int calculate_score (Media m, string search) {
		int rv = 0;
		
		rv = (int)(m.rating + (m.play_count) + ((m.last_played != 0) ? 1 : 0) +
					((search in m.title.down()) ? 2 : 0));
		
		return rv;
	}
	
	LinkedList<string> getTopScores(HashMap<string, int> table, int size) {
		LinkedList<string> rv = new LinkedList<string>();
		
		int min_top_score = -1;
		foreach(var s in table.keys) {
			if(rv.size < size) {
				rv.add(s);
				
				if(table.get(s) < min_top_score || min_top_score == -1)
					min_top_score = table.get(s);
			}
			else if(table.get(s) > min_top_score) {
				// Find current minimum and remove it
				int current_min = -1;
				string current_string = "";
				foreach(var top in rv) {
					if(table.get(top) < current_min || current_min == -1) {
						current_min = table.get(top);
						current_string = top;
					}
				}
				
				rv.remove(current_string);
				rv.add(s);
				
				// find new min_top_score
				min_top_score = table.get(rv.get(0));
				foreach(var top in rv) {
					if(table.get(top) < min_top_score)
						min_top_score = table.get(top);
				}
			}
		}
		
		return rv;
	}
	
	LinkedList<int> getTopTracks(HashMap<int, int> table, int size) {
		LinkedList<int> rv = new LinkedList<int>();
		
		int min_top_score = -1;
		foreach(var id in table.keys) {
			if(rv.size < size) {
				rv.add(id);
				
				if(table.get(id) < min_top_score || min_top_score == -1)
					min_top_score = table.get(id);
			}
			else if(table.get(id) > min_top_score) {
				// Find current minimum and remove it
				int current_min = -1;
				int current_rowid = -1;
				foreach(var top in rv) {
					if(table.get(top) < current_min || current_min == -1) {
						current_min = table.get(top);
						current_rowid = top;
					}
				}
				
				rv.remove(current_rowid);
				rv.add(id);
				
				// find new min_top_score
				min_top_score = table.get(rv.get(0));
				foreach(var top in rv) {
					if(table.get(top) < min_top_score)
						min_top_score = table.get(top);
				}
			}
		}
		
		return rv;
	}
}
