//
//  Copyright (C) 2011-2012 Tom Beckmann
//	Originally written by Tom Beckmann for Footnote
//	Edited by Scott Ringwelski (2012-04-26)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using Gtk;
using Gdk;
using Granite;
using SQLHeavy;
using Gee;

public class TaggingArea : HBox {
	public enum TagType {
		ALBUM,
		ARTIST,
		GENRE
	}
	
	public Tag genre_tag;
	public Tag artist_tag;
	public Tag album_tag;
	
	public signal void tag_remove_click(TagType type);

	public TaggingArea (){
		
	}
	
	public void set_tag(TagType type, string val) {
		Tag? tag = null;
		if(type == TagType.ALBUM)
			tag = album_tag;
		else if(type == TagType.ARTIST)
			tag = artist_tag;
		else if(type == TagType.GENRE)
			tag = genre_tag;
		
		if(val == "") {
			if(tag != null) {
				tag.visible = false;
				stdout.printf("hid tag\n");
			}
		}
		else {
			if(tag == null) {
				if(type == TagType.ALBUM) {
					album_tag = add_tag(type, val);
					tag = album_tag;
				}
				if(type == TagType.ARTIST) {
					artist_tag = add_tag(type, val);
					tag = artist_tag;
				}
				if(type == TagType.GENRE) {
					genre_tag = add_tag(type, val);
					tag = genre_tag;
				}
			}
			else
				tag.tag = val;
			
			tag.show_all();
		}
	}
	
	public Tag add_tag (TagType type, string tag){
		var t = new Tag (tag, this, type);
		pack_end (t, false, false, 5);
		reorder_child (t, 0);
		
		return t;
	}
}

public class Tag : HBox {
		private TaggingArea tagparent;
		private TaggingArea.TagType type;
		private Label _tag;
		
		public string tag {
			get { return _tag.label; }
			set {
				if(value.length > 15)
					_tag.label = value.substring(0, 14) + "...";
				else
					_tag.label = value; 
			}
		}
		
		/**
		 * create a new tag
		 * @param name The displayed name of the tag
		 */
		public Tag (string name, TaggingArea parent, TaggingArea.TagType type){
			this.tagparent = parent;
			this.type = type;
			
			this.margin = 3;
			_tag = new Label(name);
			_tag.margin = 3;
			_tag.margin_right = _tag.margin_top = 6;

			Color white;
			Color.parse ("#ffffff", out white);
			_tag.modify_bg (StateType.NORMAL, white);

			Color grey;
			Color.parse ("#5D5D5D", out grey);
			_tag.modify_fg (StateType.NORMAL, grey);
			pack_end (_tag, false, false, 0);

			var close = new EventBox ();
			close.visible_window = false;
			close.add (new Image.from_stock (Gtk.Stock.CLOSE, IconSize.MENU));
			close.button_press_event.connect (on_button_press_event);
			close.margin = 3;
			close.margin_left = close.margin_top = 6;
			pack_start (close, false, false, 0);
		}
		
		public override bool draw (Cairo.Context ctx){
			int w = this.get_allocated_width ();
			int h = this.get_allocated_height ();
			Granite.Drawing.Utilities.cairo_rounded_rectangle(ctx, 4, 4, w-5, h-5, 2);
			ctx.set_source_rgba (1.0, 1.0, 1.0, 1.0);
			ctx.fill_preserve ();
			ctx.set_source_rgba (0.2, 0.2, 0.2, 1.0);
			ctx.set_line_width (0.5);
			ctx.stroke ();
			ctx.move_to (24,4);
			ctx.line_to (24,h-1);
			ctx.stroke ();
			return base.draw (ctx);
		}

		private bool on_button_press_event(Gdk.EventButton event) {
			tagparent.tag_remove_click(this.type);
			
			return false;
		}
	}
